/*
 *  linux/include/asm-arm/arch-omap1510/dma.h
 *
 *  Copyright (C) 1997,1998 Russell King
 *  Copyright (C) 2000 Nicolas Pitre
 *  Copyright (C) 2001 RidgeRun, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#define MAX_DMA_ADDRESS		0xffffffff

#define MAX_DMA_CHANNELS	0

/*
 * All possible OMAP1510 devices a DMA channel can be attached to.
 *  See Table 5-7 in TRM.
 */
typedef enum {
        eDMANotSync,
        eExt1,
        eExt2,
        eExt3,
        eExt4,             // 5
        eExt5,
        eExt6,
        eMicroWireTx,
        eAudioTx,
        eAudioRx,
        eOpticalTx,        // 10
        eOpticalRx,
        eBluetoothTx,
        eBluetoothRx,
        eModemTx,
        eModemRx,          // 15
        eModemDataTx,
        eModemDataRx,
        eUART3Tx,
        eUART3Rx,
        eCameraRx,         // 20
        eUSBTx0,
        eUSBTx1,
        eUSBRx0,
        eUSBRx1,           // 24
} dma_device_t;

typedef enum {
        eDmaIn,
        eDmaOut
} dma_direction_t;

/*
  This allows complete control of dma channel to those that need it.
*/
typedef struct {
        u16 channel_parameters;
        u16 channel_control;
        u16 channel_interrupt;
        u16 channel_source_lower;
        u16 channel_source_upper;
        u16 channel_destn_lower;
        u16 channel_destn_upper;
        u16 channel_element_number;
        u16 channel_frame_number;
        u16 channel_frame_index;
        u16 channel_element_index;
        dma_direction_t channel_direction;
} dma_setup_t;

/*
  If we're using the iPAQ sound system, we need to remap their DMA to our DMA.
*/
#if defined(CONFIG_SOUND_SA1100) || defined(CONFIG_SOUND_SA1100_MODULE)

#define sa1100_request_dma omap1510_request_dma
#define sa1100_dma_set_callback omap1510_dma_set_callback
#define sa1100_dma_set_spin omap1510_dma_set_spin
#define sa1100_dma_queue_buffer omap1510_dma_queue_buffer
#define sa1100_dma_get_current omap1510_dma_get_current
#define sa1100_dma_stop omap1510_dma_stop
#define sa1100_dma_resume omap1510_dma_resume
#define sa1100_dma_flush_all omap1510_dma_flush_all
#define sa1100_free_dma omap1510_free_dma
#define sa1100_dma_sleep omap1510_dma_sleep
#define sa1100_dma_wakeup omap1510_dma_wakeup
#define sa1100_dma_setup omap1510_dma_setup

#endif

typedef void (*dma_callback_t)( void *buf_id, int size );

extern int omap1510_request_dma( dmach_t *channel, const char *device_id,
			       dma_device_t device );
extern int omap1510_dma_set_callback( dmach_t channel, dma_callback_t cb );
extern int omap1510_dma_set_spin( dmach_t channel, dma_addr_t addr, int size );
extern int omap1510_dma_queue_buffer( dmach_t channel, void *buf_id,
				    dma_addr_t data, int size );
extern int omap1510_dma_get_current( dmach_t channel, void **buf_id, dma_addr_t *addr );
extern int omap1510_dma_stop( dmach_t channel );
extern int omap1510_dma_resume( dmach_t channel );
extern int omap1510_dma_flush_all( dmach_t channel );
extern void omap1510_free_dma( dmach_t channel );
extern int omap1510_dma_sleep( dmach_t channel );
extern int omap1510_dma_wakeup( dmach_t channel );
extern int omap1510_dma_setup( dmach_t channel, dma_setup_t *setup );

#ifdef CONFIG_IRAM_ALLOC

#include <asm/arch/sizes.h>

// Setup a fake 1MB IRAM region in normal SDRAM.

static inline void 
__arch_adjust_zones(int node, unsigned long *size, unsigned long *holes)
{
          size[0] -= (SZ_1M >> PAGE_SHIFT);
          size[3] = (SZ_1M >> PAGE_SHIFT);
}


#endif

#endif /* _ASM_ARCH_DMA_H */

