/*
 *  linux/include/asm-arm/arch-ti925/omap1510/hardware.h
 *
 * BRIEF MODULE DESCRIPTION
 *   TI925 hardware map
 *
 * Copyright (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * Author: RidgeRun, Inc.
 *         Greg Lonnon (glonnon@ridgerun.com) or info@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __ASM_ARCH_OMAP1510_HARDWARE_H
#define __ASM_ARCH_OMAP1510_HARDWARE_H
 
#include <asm/arch/sizes.h>

/*
 * Where in virtual memory the IO devices (timers, system controllers
 * and so on)
 * ???_BASE is the virtual address
 * ???_START is the physical address
 */
#define IO_BASE			0xFFFB0000                 // VA of IO 
#define IO_SIZE			   0x40000                 // How much?
#define IO_START		0xFFFB0000                 // PA of IO

/* maps in the FPGA registers and the ETHR registers */
#define OMAP1510_FPGA_BASE      0xE8000000                 // VA
#define OMAP1510_FPGA_SIZE      SZ_4K                      // SIZE
#define OMAP1510_FPGA_START     0x08000000                 // PA   

#ifdef	CONFIG_EVM_SUPPORT_ESG
#define OMAP1510_EPLD_BASE       0xE97FF000               // VA 
#define OMAP1510_EPLD_SIZE       SZ_4K                   // size 
#define OMAP1510_EPLD_START      0x097FF000               // PA
#endif	/* CONFIG_EVM_SUPPORT_ESG */

#define OMAP1510_DSP_BASE       0xE0000000                // VA
#define OMAP1510_DSP_SIZE       0x50000                   // size
#define OMAP1510_DSP_START      0xE0000000                // PA

#define OMAP1510_DSPDARAM_BASE   0xE0000000            // VA
#define OMAP1510_DSPDARAM_SIZE   0x10000               // size
#define OMAP1510_DSPDARAM_START  0xE0000000            // PA

#define OMAP1510_DSPSARAM_BASE   0xE0010000            // VA
#define OMAP1510_DSPSARAM_SIZE   0x40000               // size
#define OMAP1510_DSPSARAM_START  0xE0010000            // PA

#define OMAP1510_DSPPDROM_BASE   0xE0FF8000            // VA
#define OMAP1510_DSPPDROM_SIZE   0x8000                // size
#define OMAP1510_DSPPDROM_START  0xE0FF8000            // PA

#define OMAP1510_DSPREG_BASE    0xE1000000                // VA
#define OMAP1510_DSPREG_SIZE    SZ_128K                   // size
#define OMAP1510_DSPREG_START   0xE1000000                // PA

#define OMAP1510_SRAM_BASE       0xD0000000                // VA
#define OMAP1510_SRAM_SIZE       ( SZ_128K + SZ_64K )      // size
#define OMAP1510_SRAM_START      0x20000000                // PA

/*
#define OMAP1510_ROMMTD_BASE     0xD8000000                   // VA
#define OMAP1510_ROMMTD_SIZE     (SZ_8M + SZ_4M + SZ_1M)      // size
#define OMAP1510_ROMMTD_START    0x00240000                   // PA
*/

// FIXME: JWi
#define OMAP1510_ROMMTD_BASE     0xD8000000                   // VA
#define OMAP1510_ROMMTD_START    0x00020000                   // PA
#define OMAP1510_ROMMTD_SIZE     (SZ_16M - OMAP1510_ROMMTD_START) // size

#define OMAP1510_RAMMTD_BASE     0xD9000000                   // VA
#define OMAP1510_RAMMTD_SIZE     (SZ_8M)                      // size
#define OMAP1510_RAMMTD_START    0x11800000                   // PA

/* JL 20.2.2002 */
#define OMAP1510_FLASH0_BASE           0xEc000000            /* VA */
#define OMAP1510_FLASH0_SIZE           SZ_32M                /*0x00200000*/
#define OMAP1510_FLASH0_START          0x00000000            /* PA */

/*
  There are 2 sets of general I/O -->
  1. GPIO (shared between ARM & DSP, configured by ARM)
  2. MPUIO which can be used only by the ARM.
  
  Base address FFFB:5000 is where the ARM accesses the MPUIO control registers
  (see 7.2.2 of the TRM for MPUIO reg definitions).
  
  Base address E101:5000 is reserved for ARM access of the same MPUIO control
  regs, but via the DSP I/O map.  This address is unavailable on 1510.
  
  Base address FFFC:E000 is where the ARM accesses the GPIO config registers
  directly via its own peripheral bus.
  
  Base address E101:E000 is where the ARM can access the same GPIO config
  registers, but the access takes place through the ARM port interface (called
  API or MPUI) via the DSP's peripheral bus (DSP I/O space).
  
  Therefore, the ARM should setup the GPIO regs thru the FFFC:E000 addresses
  instead of the E101:E000 addresses.  The DSP has only read access of the pin
  control register, so this may explain the inability to write to E101:E018.
  Try accessing pin control reg at FFFC:E018.
 */
#define OMAP1510_GPIO_BASE       0xfffce000
#define OMAP1510_GPIO_START      OMAP1510_GPIO_BASE
#define OMAP1510_GPIO_SIZE       SZ_4K

#define OMAP1510_MCBSP1_BASE     0xE1011000               // VA
#define OMAP1510_MCBSP1_SIZE     SZ_4K                    // size
#define OMAP1510_MCBSP1_START    0xE1011000               // PA

#define OMAP1510_MCBSP3_BASE     0xE1017000               // VA
#define OMAP1510_MCBSP3_SIZE     SZ_4K                    // size
#define OMAP1510_MCBSP3_START    0xE1017000               // PA

/*
 * Where's the flush address (for flushing D and I cache?)
 */
#define FLUSH_BASE              0xdf000000
#define FLUSH_BASE_PHYS		TI925_FLASH_BASE

#ifndef __ASSEMBLER__

#define PCIO_BASE		IO_BASE

/*
 * RAM definitions
 */
#define MAPTOPHYS(a)		((unsigned long)(a) - PAGE_OFFSET)
#define KERNTOPHYS(a)		((unsigned long)(&a))
#define KERNEL_BASE             (0x10008000)  
#endif

/* macro to get at IO space when running virtually */
#define IO_ADDRESS(x) ((x)) 

#define PLATFORM_ID                     0x00000045

/*
 *  TI925 memory map
 */
#define TI925_FLASH_BASE           0x00000000
#define TI925_SDRAM_BASE           0x10000000


#if 0
#define TI925_FLASH_START          TI925_FLASH_BASE
#define TI925_FLASH_SIZE           0x04000000
#define TI925_FLASH_BUSWIDTH       2
#define TI925_FLASH_SETUP_REGISTER 0x08000B00
#endif

#define OMAP1510_FPGA_ETHR_BASE 0xE8000300

/* ----------------------------------------------------------------------------
 *  OMAP1510 system registers
 * ----------------------------------------------------------------------------
 */

#define OMAP1510_UART1_BASE        0xfffb0000    /* "BLUETOOTH-UART" */
#define OMAP1510_UART2_BASE        0xfffb0800    /* "MODEM-UART" */
#if defined(CONFIG_ARCH_OMAP1509_ESG)
#define OMAP1510_UART3_BASE        0xfffce800    /* Shared MPU/DSP UART */
#else	/* !defined(CONFIG_ARCH_OMAP1509_ESG) */
#define OMAP1510_UART3_BASE        0xfffb9800    /* Shared MPU/DSP UART */
#endif	/* defined(CONFIG_ARCH_OMAP1509_ESG) */
#define OMAP1510_COM_MCBSP2_BASE   0xffff1000    /* Com McBSP2 */
#define OMAP1510_AUDIO_MCBSP_BASE  0xffff1800    /* Audio McBSP2 */
#define OMAP1510_RTC_BASE          0xffff4800    /* RTC */
#define OMAP1510_ARMIO_BASE        0xfffb5000    /* keyboard/gpio */


/*
 * Configuration Registers
 */
#define FUNC_MUX_CTRL_0         0xfffe1000
#define FUNC_MUX_CTRL_1         0xfffe1004
#define FUNC_MUX_CTRL_2         0xfffe1008

/*
 * Traffic Controller Memory Interface Registers
 */
#define TCMIF_BASE              0xfffecc00
#define IMIF_PRIO               (TCMIF_BASE + 0x00)
#define EMIFS_PRIO_REG          (TCMIF_BASE + 0x04)
#define EMIFF_PRIO_REG          (TCMIF_BASE + 0x08)
#define EMIFS_CONFIG_REG        (TCMIF_BASE + 0x0c)
#define EMIFS_CS0_CONFIG        (TCMIF_BASE + 0x10)
#define EMIFS_CS1_CONFIG        (TCMIF_BASE + 0x14)
#define EMIFS_CS2_CONFIG        (TCMIF_BASE + 0x18)
#define EMIFS_CS3_CONFIG        (TCMIF_BASE + 0x1c)
#define EMIFF_SDRAM_CONFIG      (TCMIF_BASE + 0x20)
#define EMIFF_MRS               (TCMIF_BASE + 0x24)
#define TC_TIMEOUT1             (TCMIF_BASE + 0x28)
#define TC_TIMEOUT2             (TCMIF_BASE + 0x2c)
#define TC_TIMEOUT3             (TCMIF_BASE + 0x30)
#define TC_ENDIANISM            (TCMIF_BASE + 0x34)

/*
 * LCD Panel
 */
#define TI925_LCD_BASE        0xFFFEC000        // VA of the LCD controller
#define TI925_LCD_CONTROL     (TI925_LCD_BASE)
#define TI925_LCD_TIMING0     (TI925_LCD_BASE+0x4)
#define TI925_LCD_TIMING1     (TI925_LCD_BASE+0x8)
#define TI925_LCD_TIMING2     (TI925_LCD_BASE+0xc)
#define TI925_LCD_STATUS      (TI925_LCD_BASE+0x10)
#define TI925_LCD_SUBPANEL    (TI925_LCD_BASE+0x14)

/* 
 * DMA
 */

#define OMAP1510_DMA_BASE 0xFFFED800
/* Global Register selection */
#define GLOBAL_DMA_ACCESS 1
#define NO_GLOBAL_DMA_ACCESS 0

/* Channel select field
 * NOTE: all other channels are linear, chan0 is 0, chan1 is 1, etc...
 */
#define LCD_CHANNEL 0xc

/* Register Select Field - Global Registers */
#define DMA_GCR 0
#define DMA_SUB 1

/* Register Select Field (LCD) */
#define DMA_LCD_CTRL        0
#define DMA_LCD_TOP_F1_L    1
#define DMA_LCD_TOP_F1_U    2
#define DMA_LCD_BOT_F1_L    3
#define DMA_LCD_BOT_F1_U    4

#define LCD_FRAME_MODE             (1<<0)
#define LCD_FRAME_IT_IE            (1<<1)
#define LCD_BUS_ERROR_IT_IE        (1<<2)
#define LCD_FRAME_1_IT_COND        (1<<3)
#define LCD_FRAME_2_IT_COND        (1<<4) 
#define LCD_BUS_ERROR_IT_COND      (1<<5)
#define LCD_SOURCE_IMIF            (1<<6)


/* ---------------------------------------------------------------------------
 *  OMAP1510 Interrupt Handlers
 * ---------------------------------------------------------------------------
 *
 */
#define OMAP1510_IH1_BASE          0xfffecb00
#define OMAP1510_IH2_BASE          0xfffe0000
#define OMAP1510_ITR               0x0
#define OMAP1510_MASK              0x4

#define INTERRUPT_HANDLER_BASE   OMAP1510_IH1_BASE
#define INTERRUPT_INPUT_REGISTER OMAP1510_ITR
#define INTERRUPT_MASK_REGISTER  OMAP1510_MASK


/* ---------------------------------------------------------------------------
 *  OMAP1510 FPGA
 * ---------------------------------------------------------------------------
 * 
 */

#define FPGA_BASE              0xe8000000
#define FPGA_REV_LOW           (FPGA_BASE + 0x0)   /* Revision */
#define FPGA_REV_HIGH          (FPGA_BASE + 0x1)   /* Revision */
#define FPGA_LCD_PANEL_CONTROL (FPGA_BASE + 0x2)
#define FPGA_LED_DIGIT         (FPGA_BASE + 0x3)
#define FPGA_HID_SPI           (FPGA_BASE + 0x4)
#define FPGA_POWER             (FPGA_BASE + 0x5)
#define FPGA_ISR_LO            (FPGA_BASE + 6)   /* Interrupt Status Register (Low) */
#define FPGA_ISR_HI            (FPGA_BASE + 7)   /* Interrupt Status Register (High) */
#define FPGA_IMR_LO            (FPGA_BASE + 8)   /* Interrupt Mask Register (Low) */
#define FPGA_IMR_HI            (FPGA_BASE + 9)   /* Interrupt Mask Register (High) */
#define FPGA_HOST_RESET        (FPGA_BASE + 0xa)
#define FPGA_RST               (FPGA_BASE + 0xb) /* Peripheral Reset */
#define FPGA_AUDIO             (FPGA_BASE + 0xc)
#define FPGA_DIP               (FPGA_BASE + 0xe)
#define FPGA_FPGA_IO           (FPGA_BASE + 0xf)
#define FPGA_UART1             (FPGA_BASE + 0x14)
#define FPGA_UART2             (FPGA_BASE + 0x15)
#define FPGA_OMAP1510_STATUS   (FPGA_BASE + 0x16)
#define FPGA_BOARD_REV         (FPGA_BASE + 0x18)
#define PPT_DATA               (FPGA_BASE + 0x100)
#define PPT_STATUS             (FPGA_BASE + 0x101)
#define PPT_CONTROL            (FPGA_BASE + 0x102)

#define OMAP1510_FPGA_HID_SPI 0x4

/* ---------------------------------------------------------------------------
 *  OMAP1510 TIMERS
 * ---------------------------------------------------------------------------
 * 
 */

#define OMAP1510_32kHz_TIMER_BASE 0xfffb9000

#define OMAP1510_TIMER1_BASE 0xfffec500
#define OMAP1510_TIMER2_BASE 0xfffec600
#define OMAP1510_TIMER3_BASE 0xfffec700

/* ---------------------------------------------------------------------------
 *  OMAP1510 LED
 * ---------------------------------------------------------------------------
 * 
 */
#define OMAP1510_LED_BASE 0xe8000003

/* ---------------------------------------------------------------------------
 *  OMAP1510 GPIO (only)
 * ---------------------------------------------------------------------------
 * 
 */
#define GPIO_DATA_INPUT_REG  (OMAP1510_GPIO_BASE + 0x0)
#define GPIO_DATA_OUTPUT_REG (OMAP1510_GPIO_BASE + 0x4)
#define GPIO_DIR_CONTROL_REG (OMAP1510_GPIO_BASE + 0x8)
#define GPIO_INT_CONTROL_REG (OMAP1510_GPIO_BASE + 0xc)
#define GPIO_INT_MASK_REG    (OMAP1510_GPIO_BASE + 0x10)
#define GPIO_INT_STATUS_REG  (OMAP1510_GPIO_BASE + 0x14)
#define GPIO_PIN_CONTROL_REG (OMAP1510_GPIO_BASE + 0x18)

#ifdef CONFIG_OMAPDSP // T.Koba [MSu]

/*
 *  Clock Generation and Reset Control Registers
 */
#define CLKGEN_RESET_BASE           (0xfffece00)
#define OMAP1510_ARM_CKCTL          (CLKGEN_RESET_BASE + 0x0)    
#define OMAP1510_ARM_IDLECT1        (CLKGEN_RESET_BASE + 0x4)    
#define OMAP1510_ARM_IDLECT2        (CLKGEN_RESET_BASE + 0x8)    
#define OMAP1510_ARM_EWUPCT         (CLKGEN_RESET_BASE + 0xC)    
#define OMAP1510_ARM_RSTCT1         (CLKGEN_RESET_BASE + 0x10)    
#define OMAP1510_ARM_RSTCT2         (CLKGEN_RESET_BASE + 0x14)    
#define OMAP1510_ARM_SYSST          (CLKGEN_RESET_BASE + 0x18)

/*
 * OMAP1510 DSP MMU
 */
#define OMAP1510_DSPMMU_BASE        0xfffed200
#define OMAP1510_DSPMMU_PREFETCH    (OMAP1510_DSPMMU_BASE + 0x00)
#define OMAP1510_DSPMMU_WALKING_ST  (OMAP1510_DSPMMU_BASE + 0x04)
#define OMAP1510_DSPMMU_CNTL        (OMAP1510_DSPMMU_BASE + 0x08)
#define OMAP1510_DSPMMU_FAULT_AD_H  (OMAP1510_DSPMMU_BASE + 0x0c)
#define OMAP1510_DSPMMU_FAULT_AD_L  (OMAP1510_DSPMMU_BASE + 0x10)
#define OMAP1510_DSPMMU_F_ST        (OMAP1510_DSPMMU_BASE + 0x14)
#define OMAP1510_DSPMMU_IT_ACK      (OMAP1510_DSPMMU_BASE + 0x18)
#define OMAP1510_DSPMMU_TTB_H       (OMAP1510_DSPMMU_BASE + 0x1c)
#define OMAP1510_DSPMMU_TTB_L       (OMAP1510_DSPMMU_BASE + 0x20)
#define OMAP1510_DSPMMU_LOCK        (OMAP1510_DSPMMU_BASE + 0x24)
#define OMAP1510_DSPMMU_LD_TLB      (OMAP1510_DSPMMU_BASE + 0x28)
#define OMAP1510_DSPMMU_CAM_H       (OMAP1510_DSPMMU_BASE + 0x2c)
#define OMAP1510_DSPMMU_CAM_L       (OMAP1510_DSPMMU_BASE + 0x30)
#define OMAP1510_DSPMMU_RAM_H       (OMAP1510_DSPMMU_BASE + 0x34)
#define OMAP1510_DSPMMU_RAM_L       (OMAP1510_DSPMMU_BASE + 0x38)
#define OMAP1510_DSPMMU_GFLUSH      (OMAP1510_DSPMMU_BASE + 0x3c)
#define OMAP1510_DSPMMU_FLUSH_ENTRY (OMAP1510_DSPMMU_BASE + 0x40)
#define OMAP1510_DSPMMU_READ_CAM_H  (OMAP1510_DSPMMU_BASE + 0x44)
#define OMAP1510_DSPMMU_READ_CAM_L  (OMAP1510_DSPMMU_BASE + 0x48)
#define OMAP1510_DSPMMU_READ_RAM_H  (OMAP1510_DSPMMU_BASE + 0x4c)
#define OMAP1510_DSPMMU_READ_RAM_L  (OMAP1510_DSPMMU_BASE + 0x50)

/*
 *  MPUI
 */
#define OMAP1510_MPUI_BASE             0xfffec900
#define OMAP1510_MPUI_CTRL             (OMAP1510_MPUI_BASE + 0x00)
#define OMAP1510_MPUI_DEBUG_ADDR       (OMAP1510_MPUI_BASE + 0x04)
#define OMAP1510_MPUI_DEBUG_DATA       (OMAP1510_MPUI_BASE + 0x08)
#define OMAP1510_MPUI_DEBUG_FLAG       (OMAP1510_MPUI_BASE + 0x0c)
#define OMAP1510_MPUI_STATUS           (OMAP1510_MPUI_BASE + 0x10)
#define OMAP1510_MPUI_DSP_STATUS       (OMAP1510_MPUI_BASE + 0x14)
#define OMAP1510_MPUI_DSP_BOOT_CONFIG  (OMAP1510_MPUI_BASE + 0x18)
#define OMAP1510_MPUI_DSP_API_CONFIG   (OMAP1510_MPUI_BASE + 0x1c)

/*
 *  Mailbox Registers
 */
#define OMAP1510_MAILBOX_BASE           0xfffcf000
#define OMAP1510_MAILBOX_ARM2DSP1       (OMAP1510_MAILBOX_BASE + 0x00)
#define OMAP1510_MAILBOX_ARM2DSP1b      (OMAP1510_MAILBOX_BASE + 0x04)
#define OMAP1510_MAILBOX_DSP2ARM1       (OMAP1510_MAILBOX_BASE + 0x08)
#define OMAP1510_MAILBOX_DSP2ARM1b      (OMAP1510_MAILBOX_BASE + 0x0c)
#define OMAP1510_MAILBOX_DSP2ARM2       (OMAP1510_MAILBOX_BASE + 0x10)
#define OMAP1510_MAILBOX_DSP2ARM2b      (OMAP1510_MAILBOX_BASE + 0x14)
#define OMAP1510_MAILBOX_ARM2DSP1_Flag  (OMAP1510_MAILBOX_BASE + 0x18)
#define OMAP1510_MAILBOX_DSP2ARM1_Flag  (OMAP1510_MAILBOX_BASE + 0x1c)
#define OMAP1510_MAILBOX_DSP2ARM2_Flag  (OMAP1510_MAILBOX_BASE + 0x20)

#endif

#endif
