/*
 * linux/include/asm-arm/arch-ti925/time.h
 *
 * BRIEF MODULE DESCRIPTION
 * 32kHz timer definition
 *
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *         Greg Lonnon (glonnon@ridgerun.com) or info@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#if !defined(__ASM_ARCH_OMAP1510_TIME_H)
#define __ASM_ARCH_OMAP1510_TIME_H

#include <linux/config.h>
#include <asm/system.h>
#include <asm/leds.h>
#include <asm/irq.h>

#undef DEBUG
//#define DEBUG
#ifdef DEBUG
#  define DBG(fmt, args...) printk( "%s: " fmt, __FUNCTION__ , ## args)
#  define ENTRY() DBG(":entry\n")
#else
#  define DBG(fmt, args...)
#  define ENTRY()
#endif

#if 1 // debug start

inline unsigned long  timer_read(int reg) {
        unsigned long val;
        val = (inw(IO_ADDRESS((reg) + OMAP1510_32kHz_TIMER_BASE)));
        DBG("TR: r:%d v:%x\n",reg,val);
        return val;
}
inline void timer_write(int reg,int val) {
        DBG("TW: r:%d v:%x\n",reg,val);
        outw( (val), (IO_ADDRESS( (reg) + OMAP1510_32kHz_TIMER_BASE)));
}

// Timer Registers
#define TIMER_CR     0x08
#define TIMER_TVR    0x00
#define TIMER_TCR    0x04

// Timer Control Register definition
#define TIMER_TSS    (1<<0)
#define TIMER_TRB    (1<<1)
#define TIMER_INT    (1<<2)
#define TIMER_ARL    (1<<3)

/*
 * How long is the timer interval? 100 HZ, right...
 * IRQ rate = (TVR + 1) / 32768 seconds 
 * TVR = 32768 * IRQ_RATE -1
 * IRQ_RATE =  1/100
 * TVR = 326
 */
#define TIMER_PERIOD 326 
//#define TIMER_PERIOD 0x7ff


/*
 * IRQ handler for the timer
 */
static inline void start_timer(void) {
//	timer_write(TIMER_CR,TIMER_TSS | TIMER_TRB | TIMER_INT); // start
	timer_write(TIMER_CR,TIMER_TSS | TIMER_TRB | TIMER_INT | TIMER_ARL); // start
}

static void omap1510_32kHz_timer_interrupt(int irq, 
        void *dev_id, 
        struct pt_regs *regs)
{
        ENTRY();
	do_leds();
	do_timer(regs);
	do_profile(regs);

#ifdef FPGA_HACK

        // True evil: if ethernet is up and running check and make sure its
        // bit is enabled in the FPGA interrupt mask register. If not, enable
        // it. This is a workaround for the mysterious cases where someone or
        // something is clearing this bit and bringing down our networking.
        // We check if ethernet is up by testing the reset bit in the FPGA.

        if (! (inb(FPGA_RST) & 1)) {
                unmask_irq(INT_ETHER);
        }

#endif
//        start_timer();
}

extern __inline__ void setup_timer(void)
{
        // since we don't call request_irq, we must init the structure
        ENTRY();
        timer_irq.handler = omap1510_32kHz_timer_interrupt;
	timer_irq.flags = SA_INTERRUPT;
	timer_write(TIMER_CR, 0x0);
        timer_write(TIMER_TVR,TIMER_PERIOD);
	setup_arm_irq(INT_OS_32kHz_TIMER, &timer_irq);
        start_timer();
}
#endif // REVISIT
/*
 * Set up timer interrupt, and return the current time in seconds.
 */
#if 0
#define CNTL_TIMER 0x0
#define LOAD_TIM   0x4
#define READ_TMI   0x8

//  CNTL_TIMER register bits
#define SOFT (1<<7)
#define FREE (1<<6)
#define CLOCK_ENABLE (1<<5)
#define PTV_MASK 0x7
#define PTR_SHIFT (1<<2)
#define AR (1<<1)
#define ST (1<<0)

// After programming PTV with 0, the count is 12MHz.  For 
// a clock rate of 100Hz:  100Hz * X = 12MHz 
//                         X = 12MHz/ 100Hz = 1200
#define TIMER_VALUE (12000000/100) 

static inline void start_timer(void) {
        outl(CLOCK_ENABLE | AR | ST, OMAP1510_TIMER1_BASE + CNTL_TIMER);
}

static void omap1510_timer1_interrupt(int irq, void *dev_id,
                                     struct pt_regs *regs) {

        ENTRY();
	do_leds();
	do_timer(regs);
	do_profile(regs);
        start_timer();
}

extern __inline__ void setup_timer(void) {
        ENTRY();
        timer_irq.handler = omap1510_timer1_interrupt;
	timer_irq.flags = SA_INTERRUPT;



	outl(TIMER_VALUE,OMAP1510_TIMER1_BASE + LOAD_TIM); 
	setup_arm_irq(INT_TIMER1, &timer_irq);
        start_timer();  
}
#endif

#endif
/* ---------------------------------------------------------------------------
 * Local variables:
 * c-file-style: "linux"
 * End:
 */
