/*
 *  linux/include/asm-arm/arch-omap710/hardware.h
 *
 * BRIEF MODULE DESCRIPTION
 *   OMAP710 hardware map
 *
 * Copyright (C) 2001, 2002 RidgeRun, Inc. (http://www.ridgerun.com)
 * Author: RidgeRun, Inc.
 *         Greg Lonnon (glonnon@ridgerun.com) or info@ridgerun.com
 *         Brian Mahaffy (brianm@ridgerun.com) or info@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __ASM_ARCH_710_HARDWARE_H
#define __ASM_ARCH_710_HARDWARE_H
 
#include <asm/arch/sizes.h>

/*
 * Where in virtual memory the IO devices (timers, system controllers
 * and so on)
 * ???_BASE is the virtual address
 * ???_START is the physical address
 */
#define IO_BASE			0xFFFB0000                 // VA of IO 
#define IO_SIZE			0x00040000                 // How much?
#define IO_START		0xFFFB0000                 // PA of IO

/* maps in the FPGA registers and the ETHR registers */
#define FPGA_BASE      0xE8000000                 // VA
#define FPGA_SIZE      SZ_4K                      // SIZE
#define FPGA_START     0x08000000                 // PA   
#define FPGA_ETHR_BASE 0xE8000100 /* drivers/net/smc9194.c uses this */


#define EPLD_BASE       0xEBFFF000                // VA
#define EPLD_SIZE       SZ_16K                    // size
#define EPLD_START      0x0BFFF000                // PA

#define OMAP710_ROMMTD_BASE     0xD8000000                   // VA
#define OMAP710_ROMMTD_SIZE     (SZ_8M + SZ_4M + SZ_1M)      // size
#define OMAP710_ROMMTD_START    0x00240000                   // PA

#define OMAP710_RAMMTD_BASE     0xD9000000                   // VA
#define OMAP710_RAMMTD_SIZE     (SZ_8M)                      // size
#define OMAP710_RAMMTD_START    0x10200000                   // PA


/*
 * Where's the flush address (for flushing D and I cache?)
 */
#define FLUSH_BASE              0xdf000000
#define FLUSH_BASE_PHYS		FLASH_BASE

#ifndef __ASSEMBLER__

#define PCIO_BASE		IO_BASE

/*
 * RAM definitions
 */
#define MAPTOPHYS(a)		((unsigned long)(a) - PAGE_OFFSET)
#define KERNTOPHYS(a)		((unsigned long)(&a))
#define KERNEL_BASE             (0x10008000)  
#endif

/* macro to get at IO space when running virtually */
#define IO_ADDRESS(x) ((x)) 

#define PLATFORM_ID                     0x00000045

/*
 *  memory map
 */
#define FLASH_BASE           0x00000000
#define SDRAM_BASE           0x10000000

/*
 * PERSEUS_CONF
 */
#define PERSEUS_CONF            0xfffe1000
#define PERSEUS_DEV_ID0         (PERSEUS_CONF + 0x00)
#define PERSEUS_DEV_ID1         (PERSEUS_CONF + 0x02)
#define GSM_EXT_LEAD_CONF       (PERSEUS_CONF + 0x04)
#define GSM_EXT_ARM_CONF        (PERSEUS_CONF + 0x06)
#define GSM_ASIC_CONF           (PERSEUS_CONF + 0x08)
#define GSM_IO_CONF             (PERSEUS_CONF + 0x0A)
#define PERSEUS_OSC32K_GSM_CONF (PERSEUS_CONF + 0x0C)
#define GSM_MCU_TRACE_SW        (PERSEUS_CONF + 0x0E)
#define PERSEUS_DIE_ID0         (PERSEUS_CONF + 0x10)
#define PERSEUS_DIE_ID1         (PERSEUS_CONF + 0x12)
#define PERSEUS_DIE_ID2         (PERSEUS_CONF + 0x14)
#define PERSEUS_DIE_ID3         (PERSEUS_CONF + 0x16)
/* skip 0x18 */
#define PERSEUS_MODE2           (PERSEUS_CONF + 0x1A)
#define PERSEUS_MODE1           (PERSEUS_CONF + 0x1C)
#define PERSEUS_IO_CONF0        (PERSEUS_CONF + 0x1E)
#define PERSEUS_IO_CONF1        (PERSEUS_CONF + 0x20)
#define PERSEUS_IO_CONF2        (PERSEUS_CONF + 0x22)
#define PERSEUS_IO_CONF3        (PERSEUS_CONF + 0x24)
#define PERSEUS_IO_CONF4        (PERSEUS_CONF + 0x26)
#define PERSEUS_IO_CONF5        (PERSEUS_CONF + 0x28)
#define PERSEUS_IO_CONF6        (PERSEUS_CONF + 0x2A)
#define PERSEUS_IO_CONF7        (PERSEUS_CONF + 0x2C)
#define PERSEUS_IO_CONF8        (PERSEUS_CONF + 0x2E)
#define PERSEUS_IO_CONF9        (PERSEUS_CONF + 0x30)
#define PERSEUS_AUDIO_CONF      (PERSEUS_CONF + 0x32)
/* skip 0x34 */
#define PERSEUS_OSC32K_MPU_CONF (PERSEUS_CONF + 0x36)
#define FLASH_PROTECT           (PERSEUS_CONF + 0x38)

/*
 *  Clock Generation and Reset Control Registers
 */

#define CLKGEN_RESET_BASE       (0xfffece00)
#define ARM_CKCTL               (CLKGEN_RESET_BASE + 0x0)    
#define ARM_IDLECT1             (CLKGEN_RESET_BASE + 0x4)    
#define ARM_IDLECT2             (CLKGEN_RESET_BASE + 0x8)    
#define ARM_EWUPCT              (CLKGEN_RESET_BASE + 0xC)    
#define ARM_RSTCT1              (CLKGEN_RESET_BASE + 0x10)    
#define ARM_RSTCT2              (CLKGEN_RESET_BASE + 0x14)    
#define ARM_SYSST               (CLKGEN_RESET_BASE + 0x38)    

/* DPLL Control Registers */
#define DPLL1                   0xfffecf00
#define DPLL3                   0xfffed100

#if 0
/*
 * ARM_CKCTL
 */

#define  ARM_INTHCK_SEL  (1<<14)
#define  EN_DSPCK        (1<<13)
#define  ARM_TIMXO       (1<<12)
#define  DSPMMUDIV_SHIFT (10)
#define  DSPMMUDIV_MASK  (0x3)
#define  TCDIV_SHIFT     (8)
#define  TCDIV_MASK      (0x3)
#define  DSPDIV_SHIFT    (6)
#define  DSPDIV_MASK     (0x3)
#define  ARMDIV_SHIFT    (4)
#define  ARMDIV_MASK     (0x3)
#define  ARMDIV_SHIFT    (4)
#define  ARMDIV_MASK     (0x3)
#define  LCDDIV_SHIFT    (2)
#define  LCDDIV_MASK     (0x3)
#define  PERDIV_SHIFT    (0)
#define  PERDIV_MASK     (0x3)

#define  CLK_DIV_1 0
#define  CLK_DIV_2 1
#define  CLK_DIV_4 2
#define  CLK_DIV_8 3

/*
 * ARM_IDLECT1
 */
#define SETARM_IDLE  (1<<11)
#define WKUP_MODE    (1<<10)
#define IDLTIM_ARM   (1<<9)
#define IDLAPI_ARM   (1<<8)
#define IDLDPLL_ARM  (1<<7)
#define IDLIF_ARM    (1<<6)
#define IDLHSAB_ARM  (1<<5)
#define IDLLB_ARM    (1<<4)
#define IDLLCD_ARM   (1<<3)
#define IDLPER_ARM   (1<<2)
#define IDLXORP_ARM  (1<<1)
#define IDLWDT_ARM   (1<<0)

/*
 * ARM_IDLECT2
 */

#define  NE_LBFREECK  (1<<10)
#define  EN_GPIOCK    (1<<9)
#define  DMACK_REQ    (1<<8)
#define  EN_TIMCK     (1<<7)
#define  EN_APICK     (1<<6)
#define  EN_HSABCK    (1<<5)
#define  EN_LBCK      (1<<4)
#define  EN_LCDCK     (1<<3)
#define  EN_PERCK     (1<<2)
#define  EN_XORPCK    (1<<1)
#define  EN_WDTCK     (1<<0)

#endif

/* ----------------------------------------------------------------------------
 *  system registers
 * ----------------------------------------------------------------------------
 */

#define UART1_BASE        0xfffb0000    /* UART1 */
#define UART2_BASE        0xfffb0800    /* UART2 */
#define UART3_BASE        0xfffce800  
#define COM_MCBSP2_BASE   0xffff1000    /* Com McBSP2 */
#define AUDIO_MCBSP_BASE  0xffff1800    /* Audio McBSP2 */
#define RTC_BASE          0xffff4800    /* RTC */
#define ARMIO_BASE        0xfffb5000    /* keyboard/gpio */



/*
 * LCD Panel
 */
#define OMAP710_LCD_BASE        0xFFFEC000        // VA of the LCD controller
#define OMAP710_LCD_CONTROL     (OMAP710_LCD_BASE)
#define OMAP710_LCD_TIMING0     (OMAP710_LCD_BASE+0x4)
#define OMAP710_LCD_TIMING1     (OMAP710_LCD_BASE+0x8)
#define OMAP710_LCD_TIMING2     (OMAP710_LCD_BASE+0xc)
#define OMAP710_LCD_STATUS      (OMAP710_LCD_BASE+0x10)
#define OMAP710_LCD_SUBPANEL    (OMAP710_LCD_BASE+0x14)

/* 
 * DMA
 */

#define OMAP710_DMA_BASE 0xFFFED800
/* Global Register selection */
#define GLOBAL_DMA_ACCESS 1
#define NO_GLOBAL_DMA_ACCESS 0

/* Channel select field
 * NOTE: all other channels are linear, chan0 is 0, chan1 is 1, etc...
 */
#define LCD_CHANNEL 0xc

/* Register Select Field - Global Registers */
#define DMA_GCR 0
#define DMA_SUB 1

/* Register Select Field (LCD) */
#define DMA_LCD_CTRL        0
#define DMA_LCD_TOP_F1_L    1
#define DMA_LCD_TOP_F1_U    2
#define DMA_LCD_BOT_F1_L    3
#define DMA_LCD_BOT_F1_U    4

/* REVISIT gdl we need to rewrite our own DMA stuff */
#if 0
/* LCD DMA */
#define DMA_LCD_CTRL       (OMAP710_DMA_BASE + 0x34)
#define DMA_LCD_TOP_F1_L   (OMAP710_DMA_BASE + 0x38)
#define DMA_LCD_TOP_F1_U   (OMAP710_DMA_BASE + 0x3c)
#define DMA_LCD_BOT_F1_L   (OMAP710_DMA_BASE + 0x40)
#define DMA_LCD_BOT_F1_U   (OMAP710_DMA_BASE + 0x44)
#define DMA_LCD_TOP_F2_L   (OMAP710_DMA_BASE + 0x48)
#define DMA_LCD_TOP_F2_U   (OMAP710_DMA_BASE + 0x4c)
#define DMA_LCD_BOT_F2_L   (OMAP710_DMA_BASE + 0x50)
#define DMA_LCD_BOT_F2_U   (OMAP710_DMA_BASE + 0x54)
#endif 


#define LCD_FRAME_MODE             (1<<0)
#define LCD_FRAME_IT_IE            (1<<1)
#define LCD_BUS_ERROR_IT_IE        (1<<2)
#define LCD_FRAME_1_IT_COND        (1<<3)
#define LCD_FRAME_2_IT_COND        (1<<4) 
#define LCD_BUS_ERROR_IT_COND      (1<<5)
#define LCD_SOURCE_IMIF            (1<<6)


/* ---------------------------------------------------------------------------
 *  Interrupt Handlers
 * ---------------------------------------------------------------------------
 * 
 */


#define IH1_BASE          0xfffecb00
#define IH2_BASE          0xfffe0000
#define ITR               0x0
#define MASK              0x4

#define INTERRUPT_HANDLER_BASE   IH1_BASE
#define INTERRUPT_INPUT_REGISTER ITR
#define INTERRUPT_INPUT_REGISTER ITR
#define INTERRUPT_MASK_REGISTER  MASK


/* ---------------------------------------------------------------------------
 *  OMAP710 FPGA
 * ---------------------------------------------------------------------------
 * 
 */

//#define FPGA_BASE      0xE8000000                 // VA
#define FPGA_REV_LOW           (FPGA_BASE + 0x0)   /* Revision */
#define FPGA_REV_HIGH          (FPGA_BASE + 0x1)   /* Revision */
#define FPGA_LCD_PANEL_CONTROL (FPGA_BASE + 0x2)
#define FPGA_LED_DIGIT         (FPGA_BASE + 0x3)
#define FPGA_ISR_LO            (FPGA_BASE + 6)   /* Interrupt Status Register (Low) */
#define FPGA_ISR_HI            (FPGA_BASE + 7)   /* Interrupt Status Register (High) */
#define FPGA_IMR_LO            (FPGA_BASE + 8)   /* Interrupt Mask Register (Low) */
#define FPGA_IMR_HI            (FPGA_BASE + 9)   /* Interrupt Mask Register (High) */
#define FPGA_CONTROL           (FPGA_BASE + 0xd)


#endif //__ASM_ARCH_710_HARDWARE_H
