/*
 * linux/include/asm-arm/arch-ti925/omap1510/serial.h
 *
 * BRIEF MODULE DESCRIPTION
 * serial definitions
 *
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *         Greg Lonnon (glonnon@ridgerun.com) or info@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ASM_ARCH_OMAP1510_SERIAL_H
#define __ASM_ARCH_OMAP1510_SERIAL_H

#include <asm/arch/hardware.h>
#include <asm/irq.h>

#define BASE_BAUD (750000)

#define RS_TABLE_SIZE	2

#define STD_COM_FLAGS (ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST)

/* I added this to make it nice and obvious which port is used for printk's
   and as the initial console (the same port will serve as both -- which is as
   it should be). This is an index into the table laid out by 
   STD_SERIAL_PORT_DEFNS below.
 */
#define SERIAL_CONSOLE_PORT 0 /* 1 */

/* I added this to prevent apps from trying to use 115200 baud on the ports
   which don't support it (UART1 & UART2). This limit doesn't apply to UART3,
   and the serial driver was hacked accordingly (see change_speed() in 
   serial.c).
   (gjm)
 */
#ifdef CONFIG_UART_FIX_ESG	/* For UART1 & 2 by Y.Hirase(NRC/Tokyo) */
#define MAX_BAUD_RATE 115200
#else	/* For UART1 & 2 by Y.Hirase(NRC/Tokyo) */
#define MAX_BAUD_RATE 57600
#endif	/* For UART1 & 2 by Y.Hirase(NRC/Tokyo) */

/* Note that for UART3 I've doubled the base baud rate. For UART3 decimal 13 is
   115200 baud, whereas for UART1 (and, I assume, UART2) it's 57600.
   (gjm)
 */

/*
* UART configuration support by Y.Hirase(NRC/Tokyo)
*/
#if defined(CONFIG_UART_ASSIGN_TTY_ESG)

/* tts/0 definition */
#ifdef	CONFIG_UART_ASSIGN_TTYS0_UART1_ESG
#define	STD_SERIAL_PORT_DEFN_TTYS0	STD_SERIAL_PORT_DEFN_UART1
#elif	defined(CONFIG_UART_ASSIGN_TTYS0_UART2_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS0	STD_SERIAL_PORT_DEFN_UART2
#elif	defined(CONFIG_UART_ASSIGN_TTYS0_UART3_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS0	STD_SERIAL_PORT_DEFN_UART3
#elif	defined(CONFIG_UART_ASSIGN_TTYS0_NONE_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS0
#else
#define	STD_SERIAL_PORT_DEFN_TTYS0
#endif	CONFIG_UART_ASSIGN_TTYS0_UART1_ESG

/* tts/1 definition */
#ifdef	CONFIG_UART_ASSIGN_TTYS1_UART1_ESG
#define	STD_SERIAL_PORT_DEFN_TTYS1	STD_SERIAL_PORT_DEFN_UART1
#elif	defined(CONFIG_UART_ASSIGN_TTYS1_UART2_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS1	STD_SERIAL_PORT_DEFN_UART2
#elif	defined(CONFIG_UART_ASSIGN_TTYS1_UART3_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS1	STD_SERIAL_PORT_DEFN_UART3
#elif	defined(CONFIG_UART_ASSIGN_TTYS1_NONE_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS1
#else
#define	STD_SERIAL_PORT_DEFN_TTYS1
#endif	CONFIG_UART_ASSIGN_TTYS1_UART1_ESG

/* tts/2 definition */
#ifdef	CONFIG_UART_ASSIGN_TTYS2_UART1_ESG
#define	STD_SERIAL_PORT_DEFN_TTYS2	STD_SERIAL_PORT_DEFN_UART1
#elif	defined(CONFIG_UART_ASSIGN_TTYS2_UART2_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS2	STD_SERIAL_PORT_DEFN_UART2
#elif	defined(CONFIG_UART_ASSIGN_TTYS2_UART3_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS2	STD_SERIAL_PORT_DEFN_UART3
#elif	defined(CONFIG_UART_ASSIGN_TTYS2_NONE_ESG)
#define	STD_SERIAL_PORT_DEFN_TTYS2
#else
#define	STD_SERIAL_PORT_DEFN_TTYS2
#endif	CONFIG_UART_ASSIGN_TTYS2_UART1_ESG

#else	/* !defined(CONFIG_UART_ASSIGN_TTY_ESG) */

#define	STD_SERIAL_PORT_DEFN_TTYS0	STD_SERIAL_PORT_DEFN_UART1
#define	STD_SERIAL_PORT_DEFN_TTYS1	STD_SERIAL_PORT_DEFN_UART3
#define	STD_SERIAL_PORT_DEFN_TTYS2

#endif	/* defined(CONFIG_UART_ASSIGN_TTY_ESG) */

#define	STD_SERIAL_PORT_DEFN_UART1 \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: INT_UART1, \
	flags: STD_COM_FLAGS, \
	type: PORT_8250, \
	iomem_base: (u8*)OMAP1510_UART1_BASE, \
	iomem_reg_shift: 2, \
	io_type: SERIAL_IO_MEM \
	},

#define	STD_SERIAL_PORT_DEFN_UART2 \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: INT_UART3, \
	flags: STD_COM_FLAGS, \
	type: PORT_UNKNOWN, \
	iomem_base: (u8*)OMAP1510_UART2_BASE, \
	iomem_reg_shift: 2, \
	io_type: SERIAL_IO_MEM \
	},

#if defined(CONFIG_ARCH_ORCA)
#define	STD_SERIAL_PORT_DEFN_UART3 \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: INT_UART3, \
	flags: STD_COM_FLAGS, \
	type: PORT_8250, \
	iomem_base: (u8*)OMAP1510_UART3_BASE, \
	iomem_reg_shift: 2, \
	io_type: SERIAL_IO_MEM \
	},
#elif !defined(CONFIG_ARCH_OMAP1509_ESG)
#define	STD_SERIAL_PORT_DEFN_UART3 \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: INT_UART3, \
	flags: STD_COM_FLAGS, \
	type: PORT_8250, \
	iomem_base: (u8*)OMAP1510_UART3_BASE, \
	iomem_reg_shift: 2, \
	io_type: SERIAL_IO_MEM \
	},
#else
#define	STD_SERIAL_PORT_DEFN_UART3 \
	{  \
	magic: 0, \
        baud_base: (BASE_BAUD * 2), \
	port: 0, \
	irq: INT_UART3, \
	flags: STD_COM_FLAGS, \
	type: PORT_UNKNOWN, \
	iomem_base: (u8*)OMAP1510_UART3_BASE, \
	iomem_reg_shift: 0, \
	io_type: SERIAL_IO_MEM \
	},
#endif

#define STD_SERIAL_PORT_DEFNS \
	STD_SERIAL_PORT_DEFN_TTYS0 \
	STD_SERIAL_PORT_DEFN_TTYS1 \
	STD_SERIAL_PORT_DEFN_TTYS2

#define EXTRA_SERIAL_PORT_DEFNS

#endif
